#!/usr/bin/env ruby
# encoding: utf-8
# ruby: 1.9
require 'serialport'
require './telegram'

# print debug info
DEBUG = true
# connect to PC
pc_serial = SerialPort.open("/dev/ttyUSB0",{ baud: 57600, databits: 8, parity: SerialPort::ODD, stop_bit: 1, flow_control: SerialPort::NONE})

while true
  activities = IO.select([pc_serial])
  activities[0].each do |activity|
    data = activity.readpartial(16+1+5)
    line = data.unpack("C*").collect { |b| sprintf("%02X ",b) }.join
    if activity==pc_serial then
      puts "> "+line if DEBUG
      begin
        telegram_in = Telegram.parse(data)
        if telegram_in then
          puts "> "+telegram_in.to_s[2..-1]
          telegram_out = case telegram_in.object
            when 0 # device type
              Telegram.new(telegram_in.object,"simulator".bytes.to_a+[0x00],false)
            when 8 # manufacturer
              Telegram.new(telegram_in.object,"EA".bytes.to_a+[0x00],false)
            when 1, 6, 9 # article no., serial no., software version
              Telegram.new(telegram_in.object,"42".bytes.to_a+[0x00],false)
            when 2,3,4 # nominal voltage, current, power
              Telegram.new(telegram_in.object,[42.0].pack("g").bytes.to_a,false)
            when 19 # device class
              Telegram.new(telegram_in.object,[0x00,0x10],false)
            when 54,150 # power supply control, unlock code
              Telegram.new(0xff,[0x00],false)
            else # error: object not defined
              Telegram.new(0xff,[0x07],false)
          end
          if telegram_out then
            puts "< "+telegram_out.to_s[2..-1] # display message
            pc_serial.write(telegram_out.pack) # send message
          end
        else
          pc_serial.baud = (pc_serial.baud==57600 ? 115200 : 57600)
          pc_serial.parity = SerialPort::ODD
          puts "could not parse message: switching baudrate to #{pc_serial.baud} bps"
		end
      rescue Exception => e
        puts e.to_s
      end
    else
      raise "unknown source"
    end
  end
end

